local _FUNCS = {

} 

local _TIMERS = {
	
} 

local null = function() end

-- DREAMS are special functions that replace the love callbacks during a sleep timer 
-- (note that dreams update is called after Sleep.update, instead of replacing it)
_DREAMS = {update = null, keypressed = null, keyreleased = null,
mousedpressed = null, mousereleased = null, mousemoved = null, wheelmoved = null}

local remove = table.remove
--[[
Sleep makes the corresponding love callbacks sleep
Sleep does this: 
		1. Saves the original love callback data
		2. Overwrite the love callback with a dream function or a function that does nothing
		3. Create a timer and updates it every frame
		4. When the timer ends it overwrites the love callback with the saved data

After this time is up, they will be work again

Use this to run sleep 

	--# main
	Sleep = require "sleep"

	function love.update(dt)
		Sleep.update(dt)
	end

	-- 	You can call these functions
	-- 	Sleep.love.keypressed
	-- 	Sleep.love.keyreleased
	-- 	Sleep.love.mousedpressed
	-- 	Sleep.love.mousedreleased
	-- 	Sleep.love.mousedmoved
	-- 	Sleep.love.wheelmoved
	-- 	Sleep.love.update
]]


--[[
	Ideas:
	-- 	Sleep.love could be given temp functions
	-- 	so you temporarily overwrite instead of just putting them to sleep

	-- 	multi threading; but i have never used them 
	-- 	same for coroutines

	Bugs:
		Your function which replaced a normal love function gets overwritten
			--# main
			Sleep.love.keypressed(2) -- overwrite keypressed with a null function
			love.keypressed = new_keypressed --overwrite keypressed with a function that does something
			-- 2 seconds later the timer ends, keypressed is overwritten a third time
		
		note that if Sleep.love.update is called, Sleep.update replaces love.update instead of null
]]

local Sleep = {}


local function timer(duration, callback)
	return {duration=duration,elapsed=0,callback=callback}
end

-- call Sleep.update(dt) in love.update; automatically called when you use Sleep.love.update
function Sleep.update(dt)
	
	assert(dt, "Sleep.update(dt): dt does not exist!")
	for k,timer in pairs(_TIMERS) do
		timer.elapsed = timer.elapsed + dt
		if timer.duration <= timer.elapsed then
			timer.callback()
		end
	end

	_DREAMS.update(dt)
end


Sleep.love = {}

function Sleep.love.keypressed(time, dream)
	assert(time, "argument 1: 'time' is not a number!")
	if dream then assert(type(dream)=='function',"dream must be a function") end
	if _FUNCS.keypressed then
		_TIMERS.keypressed.duration = time
	else
		_FUNCS.keypressed = love.keypressed
		_TIMERS.keypressed = timer(time, 
			function() 
				love.keypressed = _FUNCS.keypressed
				_FUNCS.keypressed = nil
				_TIMERS.keypressed = nil
				_DREAMS.keypressed = null
			end)
		love.keypressed = dream or null
	end
end

function Sleep.love.keyreleased(time, dream)
	assert(time, "argument 1: 'time' is not a number!")
	if dream then assert(type(dream)=='function',"dream must be a function") end

	if _FUNCS.keyreleased then
		_TIMERS.keyreleased.duration = time
	else
		_FUNCS.keyreleased = love.keyreleased
		_TIMERS.keyreleased = timer(time, 
				function() 
					love.keyreleased = _FUNCS.keyreleased
					_FUNCS.keyreleased = nil
					_TIMERS.keyreleased = nil
					_DREAMS.keyreleased = null
				end)
		love.keyreleased = dream or null

	end
end

function Sleep.love.mousepressed(time, dream)
	assert(time, "argument 1: 'time' is not a number!")
	if dream then assert(type(dream)=='function',"dream must be a function") end

	if _FUNCS.mousepressed then
		_TIMERS.mousepressed.duration = time
	else
		_FUNCS.mousepressed = love.mousepressed
		_TIMERS.mousepressed = timer(time, 
				function() 
					love.mousepressed = _FUNCS.mousepressed
					_FUNCS.mousepressed = nil
					_TIMERS.mousepressed = nil
					_DREAMS.mousepressed = null
				end)
		love.mousepressed = dream or null
	end
end

function Sleep.love.mousereleased(time, dream)
	assert(time, "argument 1: 'time' is not a number!")
	if dream then assert(type(dream)=='function',"dream must be a function") end

	if _FUNCS.mousereleased then
		_TIMERS.mousereleased.duration = time
	else
		_FUNCS.mousereleased = love.mousereleased
		_TIMERS.mousereleased = timer(time, 
				function() 
					love.mousereleased = _FUNCS.mousereleased
					_FUNCS.mousereleased = nil
					_TIMERS.mousereleased = nil
					_DREAMS.mousereleased = null
				end)
		love.mousereleased = dream or null
	end
end

function Sleep.love.mousemoved(time, dream)
	assert(time, "argument 1: 'time' is not a number!")
	if dream then assert(type(dream)=='function',"dream must be a function") end

	if _FUNCS.mousemoved then
		_TIMERS.mousemoved.duration = time
	else
		_FUNCS.mousemoved = love.mousemoved
		_TIMERS.mousemoved = timer(time, 
				function() 
					love.mousemoved = _FUNCS.mousemoved
					_FUNCS.mousemoved = nil
					_TIMERS.mousemoved = nil
					_DREAMS.mousemoved = null
				end)
		love.mousemoved = dream or null
	end
end

function Sleep.love.wheelmoved(time, dream)
	assert(type(time)=='number', "Sleep.love:'time' was not a number!")
	if dream then assert(type(dream)=='function',"dream must be a function") end

	if _FUNCS.wheelmoved then
		_TIMERS.wheelmoved.duration = time
	else
		_FUNCS.wheelmoved = love.wheelmoved
		_TIMERS.wheelmoved = timer(time, 
				function() 
					love.wheelmoved = _FUNCS.wheelmoved
					_FUNCS.wheelmoved = nil
					_TIMERS.wheelmoved = nil
					_DREAMS.wheelmoved = null
				end)
		love.wheelmoved = dream or null
	end
end

-- Sleep.love.update replaces love.update with Sleep.update instead of null
function Sleep.love.update(time, dream)
	assert(time, "argument 1: 'time' is not a number!")
	if dream then assert(type(dream)=='function',"dream must be a function") end

	if _FUNCS.update then
		_TIMERS.update.duration = time
	else
		_FUNCS.update = love.update
		_TIMERS.update = timer(time, 
				function() 
					love.update = _FUNCS.update
					_FUNCS.update = nil
					_TIMERS.update = nil
					_DREAMS.update = null
				end)
		love.update = dream or Sleep.update
	end
end

function Sleep.forceQuit()
	local _FUNCS = {} 
	local _TIMERS = {} 
	_DREAMS = {update = null, keypressed = null, keyreleased = null,
	mousedpressed = null, mousereleased = null, mousemoved = null, wheelmoved = null}
end

return Sleep